package it.finanze.dogane.accise.aautotrasportatori.model;

import it.finanze.dogane.accise.aautotrasportatori.util.AnniDichiarazione;
import it.finanze.dogane.accise.model.AbstractRecord;

import java.io.IOException;
import java.io.InputStream;
import java.math.BigInteger;

public abstract class AutotrasportatoriRecord extends AbstractRecord {
	
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	
	private static final String valoreFisso = "CAR100";
	private BigInteger annoDic = BigInteger.ZERO;
	private String partitaIva = "";
	private BigInteger progPartitaIva = BigInteger.valueOf(0);
	private BigInteger tipoRecord = BigInteger.valueOf(0);
	
	private static final int dimValoreFisso = 6;
	private static final int dimAnnoDic =4;
	private static final int dimPartitaIvaItaliana =12;
	private static final int dimProgPartitaIva =2;
	private static final int dimTipoRecord =2;
	
	/**
	 * Se true, significa che tutti i record del quadro B
	 * hanno i litri a zero.	
	 */
	public static boolean QUADRO_B_ZERO = true;  	
	

	public void impostaCampiComuni(AbstractRecord in) {
		AutotrasportatoriRecord erin = (AutotrasportatoriRecord)in;
		partitaIva = erin.getPartitaIva();
		progPartitaIva = erin.getProgPartitaIva();
		annoDic = erin.getAnnoDic();
	}



	public void scriviOggetto(java.io.OutputStream out) throws IOException {
		
		super.scriviOggetto(out);
		
		out.write(normalizzaCampo(valoreFisso, dimValoreFisso));
		out.write(normalizzaCampo(annoDic, dimAnnoDic));
		out.write(normalizzaCampo(partitaIva, dimPartitaIvaItaliana));
		out.write(normalizzaCampo(progPartitaIva, dimProgPartitaIva));
		out.write(normalizzaCampo(tipoRecord, dimTipoRecord));
		
	}
	
	public void leggiOggetto(InputStream in) throws IOException {

		super.leggiOggetto(in);
		
		readFieldString(in, dimValoreFisso);
		annoDic = readFieldBigInteger(in, dimAnnoDic);
		try {
			
			//MAC 28/05/2012 - MAC APERTURA DICHIARAZIONI ANNI PRECEDENTI
			/*Verificare nel primo record il valore del campo anno. 
			Se l'anno  minore del 2008 inviare la segnalazione BLOCCANTE:
			'Impossibile aprire la dichiarazione di anni precedenti al 2008'*/
			String annoCambiamentoTracciato = AnniDichiarazione.ANNO_CAMBIAMENTO_TRACCIATO.anno;
			if (annoDic.intValue() < Integer.parseInt(annoCambiamentoTracciato)) {
				String msgErr = "Impossibile aprire la dichiarazione di anni precedenti al " + annoCambiamentoTracciato;
				throw new IOException(msgErr);
			}
			
				
		} catch (NumberFormatException nfEx) {
			throw new IOException("Impossibile Aprire il File: File Danneggiato. (Err.Code: NFE)");
		}
		
		partitaIva = readFieldString(in, dimPartitaIvaItaliana);
		progPartitaIva = readFieldBigInteger(in, dimProgPartitaIva);
		tipoRecord = readFieldBigInteger(in, dimTipoRecord);	
							
	}

	/**
	 * @return
	 */
	public static String getValoreFisso() {
		return valoreFisso;
	}


	/**
	 * @return
	 */
	
	/**
	 * @return
	 */
	public static int getDimAnnoDic() {
		return dimAnnoDic;
	}

	/**
	 * @return
	 */
	public static int getDimPartitaIvaItaliana() {
		return dimPartitaIvaItaliana;
	}

	/**
	 * @return
	 */
	public static int getDimProgPartitaIva() {
		return dimProgPartitaIva;
	}

	/**
	 * @return
	 */
	public static int getDimTipoRecord() {
		return dimTipoRecord;
	}

	/**
	 * @return
	 */
	public static int getDimValoreFisso() {
		return dimValoreFisso;
	}


	/**
	 * @return
	 */
	public BigInteger getProgPartitaIva() {
		return progPartitaIva;
	}

	/**
	 * @return
	 */
	public BigInteger getTipoRecord() {
		return tipoRecord;
	}

	/**
	 * @param integer
	 */
	public void setProgPartitaIva(BigInteger integer) {
		progPartitaIva = integer;
	}

	/**
	 * @param integer
	 */
	public void setTipoRecord(BigInteger integer) {
		tipoRecord = integer;
	}

	/**
	 * @return
	 */
	public BigInteger getAnnoDic() {
		return annoDic;
	}

	/**
	 * @param integer
	 */
	public void setAnnoDic(BigInteger integer) {
		annoDic = integer;
	}

	/**
	 * @return
	 */
	public String getPartitaIva() {
		return partitaIva;
	}

	/**
	 * @param string
	 */
	public void setPartitaIva(String string) {
		partitaIva = string;
	}
	
	//implementato nelle sottoclassi
	public Object[] getQuadroToArray(){return null;}



	

}
