package it.finanze.dogane.accise.aautotrasportatori.model;

import it.finanze.dogane.accise.aautotrasportatori.ui.AutotrasportatoriFrame;
import it.finanze.dogane.accise.aautotrasportatori.ui.quadri.Quadro;
import it.finanze.dogane.accise.aautotrasportatori.util.GenericUtils;
import it.finanze.dogane.accise.aautotrasportatori.util.QuadroAutotrasportatoriUtil;
import it.finanze.dogane.accise.model.AbstractRecord;
import it.finanze.dogane.accise.model.Prospect;
import it.finanze.dogane.accise.risorse.Risorse;

import it.finanze.dogane.accise.util.ComparatorListModel;
import it.finanze.dogane.accise.util.QuadroUtil;
import it.finanze.dogane.accise.util.ResourceAccess;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.io.Serializable;
import java.math.BigDecimal;
import java.math.BigInteger;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

public class ProspettoConsumo implements Serializable, Prospect {
	
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	
	private ResourceAccess resourceAccess = new ResourceAccess(this);
	private Frontespizio frontespizio =  new Frontespizio();
	private List<AutotrasportatoriRecord> listaRecord = new LinkedList<AutotrasportatoriRecord>();
	private List<AutotrasportatoriRecord> listaDatiContabili = new ArrayList<AutotrasportatoriRecord>();
	private int[] listaAliquote = new int[AutotrasportatoriFrame.getNumeroAliquote()];
	
	
	public void scriviOggetto(java.io.OutputStream out) throws IOException {
		
		updateRecordNumbers();
		frontespizio.scriviOggetto(out);
		
		//solo i dati contabili valorizzati
		int riga = 0;
		for (int i = 0; i < listaAliquote.length; i++) {
			
			riga++;						
		  	BigInteger litri = frontespizio.getArrayLitriConsumati()[i];
		  	if(litri.toString().equalsIgnoreCase("") || litri.toString().equalsIgnoreCase("0") )
		  		continue;		  	
				
			BigDecimal importi = frontespizio.getArrayImportoACredito()[i]; 
		    DatiContabiliRecord record = new DatiContabiliRecord(litri,importi,new BigInteger(""+riga));
			record.setTipoRecord(new BigInteger(""+RecordFactory.DATI_CONTABILI_TYPE));
			record.impostaCampiComuni(frontespizio);
			record.scriviOggetto(out);
		}
		
		Collections.sort(listaRecord, new ComparatorListModel());
		Iterator<AutotrasportatoriRecord> it = listaRecord.iterator();
		while(it.hasNext()) {
			AutotrasportatoriRecord record = (AutotrasportatoriRecord)it.next();
			record.impostaCampiComuni(frontespizio);
			record.scriviOggetto(out);
		}
		
		AutotrasportatoriRecord.QUADRO_B_ZERO = true;
	}

	public void leggiOggetto(java.io.InputStream in) throws IOException {

		frontespizio = new Frontespizio();
		frontespizio.initArrayLitriImporti(AutotrasportatoriFrame.getNumeroAliquote());
		listaRecord.clear();	

		frontespizio.leggiOggetto(in);				
		
		BufferedReader reader = new BufferedReader(new InputStreamReader(in));
		AutotrasportatoriRecord record = RecordFactory.readNextRecord(reader);
		
		boolean isPeriodoDiverso = frontespizio.isPeriodoDiverso();
		clearRecordFrontespizio(isPeriodoDiverso, frontespizio);
		
		while(record != null) {
			
			if (!(record instanceof AutotrasportatoriInvalidRecord))			
				if (!(record instanceof AutotrasportatoriQuadroA) ||
					((record instanceof AutotrasportatoriQuadroA)
				    && (record!=null))
				)
				
				
				//MAC 28/05/2012 - MAC APERTURA DICHIARAZIONI ANNI PRECEDENTI
				if(record instanceof DatiContabiliRecord) {
					clearRecordDatoContabile(isPeriodoDiverso, (DatiContabiliRecord) record);
					listaDatiContabili.add(record);
				} else {
					clearRecordQuadri(isPeriodoDiverso, record);
					listaRecord.add(record);
				}
				
			record = RecordFactory.readNextRecord(reader);
			
		}//end while
		
		reader.close();
	}

	/**
	 * Se periodo  diverso dal valore del campo periodo nel file di servizio PERIODI.TXT,
	 * azzera i dati contabili(o DATI IMPRESA, tipo record 00): 
	 * litri consumati ed importi a credito.
	 * 
	 * 
	 * @param isPeriodoDiverso
	 * @param record, tipo record 00	
	 */
	private final void clearRecordDatoContabile(boolean isPeriodoDiverso, DatiContabiliRecord record) {
		if (isPeriodoDiverso) {
			record.setTotaleLitri(BigInteger.valueOf(0));
			record.setTotaleImportoCredito(BigDecimal.valueOf(0));
		}
	}
	
	
	/**
	 * Se periodo  diverso dal valore del campo periodo nel file di servizio PERIODI.TXT,
	 * azzera i dati della riga totale: litri consumati ed importi a credito.
	 * 
	 * 
	 * @param isPeriodoDiverso
	 * @param record, tipo record 00
	 */
	private final void clearRecordFrontespizio(boolean isPeriodoDiverso, Frontespizio record) {
		if (isPeriodoDiverso) {
			record.setTotaleLitriConsumati(BigInteger.valueOf(0));
			record.setTotaleImportCredito(BigDecimal.valueOf(0));
		}
	}
	
	/**
	 * Se periodo  diverso dal valore del campo periodo nel file di servizio PERIODI.TXT,
	 * azzera alcune colonne dei quadri A e B.
	 * 
	 * 
	 * @param isPeriodoDiverso
	 * @param record
	 */
	private final void clearRecordQuadri(boolean isPeriodoDiverso, AutotrasportatoriRecord record) {
		if (isPeriodoDiverso) {
			
			if (record instanceof AutotrasportatoriQuadroA) {		
				clearRecordQuadroA((AutotrasportatoriQuadroA) record);
			} else if (record instanceof AutotrasportatoriQuadroB) {		
				clearRecordQuadroB((AutotrasportatoriQuadroB) record);
			}  else if (record instanceof AutotrasportatoriQuadroC) {		
				//per adesso nulla
				//QUADRO C (tipo record 30) visualizzare tutti i dati come nell'elaborazione normale
			}
			
		}//end if isPeriodoDiverso		
	}
	
	
	/**
	 * Azzera le seguenti colonne del Quadro A-n(tipo record 10):
	 * numero fatture, numero schede, litri consumati, km percorsi, importo.
	 * Reimposta data inizio possesso e data fine possesso con i dati presi
	 * dal file PERIODI.TXT
	 * 
	 * 
	 * @param record, tipo record 10
	 */
	private void clearRecordQuadroA(AutotrasportatoriQuadroA record) {		
	
		List listDatiContabili = Risorse.getInstance().getListDatiContabili();
		final int DATI_QUADRO_A1 = 0;
		DatiContabiliObject element = (DatiContabiliObject) listDatiContabili.get(DATI_QUADRO_A1);//quadro A-1
			
		record.setLitriConsumati(BigInteger.valueOf(0));
		record.setImporto(BigDecimal.valueOf(0));
		record.setChilometriPercorsi(BigInteger.valueOf(0));
		record.setNumeroSchede(BigInteger.valueOf(0));
		record.setNumeroFatture(BigInteger.valueOf(0));
			
		String periodoDa = GenericUtils.getSringDate(element.getPeriodoDa());
		String periodoA = GenericUtils.getSringDate(element.getPeriodoA());
			
		record.setDataInizioPossesso(new BigInteger(periodoDa));
		record.setDataFinePossesso(new BigInteger(periodoA));			
		
	}
	
	
	/**
	 * Azzera alcune colonne del Quadro B(tipo record 20).
	 * Se tipo serbatoio  uguale ad A (sup 10 mc), visualizza solo la colonna codice ditta;
	 * Se tipo serbatoio  uguale B (inf 10 mc), visualizza solo le colonne provincia, comune, indirizzo, capacit
	 * 
	 * 
	 * @param record, tipo record 20
	 */
	private void clearRecordQuadroB(AutotrasportatoriQuadroB record) {		
		
		String tipoSerbatoio = record.getTipoSerbatoio();
		if (tipoSerbatoio.equalsIgnoreCase("A")) {//sup 10 mc
			record.setSuperioreTotaleLitri(BigInteger.valueOf(0));
			record.setSuperioreTotaleImporto(BigDecimal.valueOf(0));
			record.setSuperioreNumeroFatture(BigInteger.valueOf(0));			
		} else if (tipoSerbatoio.equalsIgnoreCase("B")) {//inf 10 mc
			record.setInferioreTotaleLitri(BigInteger.valueOf(0));
			record.setInferioreTotaleImporto(BigDecimal.valueOf(0));
			record.setInferioreNumeroFatture(BigInteger.valueOf(0));
		}
	}
	
	
	 public static BigInteger getLitriValue (AbstractRecord record) {
			
			BigInteger litri = BigInteger.valueOf(0);
			
			if (record instanceof AutotrasportatoriQuadroA) {
				litri = ((AutotrasportatoriQuadroA) record).getLitriConsumati();
			} else if (record instanceof AutotrasportatoriQuadroB) {
				
				AutotrasportatoriQuadroB recordB = ((AutotrasportatoriQuadroB) record);
				String tipoSerbatoio = recordB.getTipoSerbatoio();
				if (tipoSerbatoio.equalsIgnoreCase("A")) {//sup 10 mc
					litri = recordB.getSuperioreTotaleLitri();						
				} else if (tipoSerbatoio.equalsIgnoreCase("B")) {//inf 10 mc
					litri = recordB.getInferioreTotaleLitri();
				}
				
			} else if (record instanceof AutotrasportatoriQuadroC) {
				return BigInteger.valueOf(1);
			}
			
			return litri;
			
		}

		
		public static BigInteger countRecord(List listaRecords, BigInteger tipoRecord) {
			int counter = 0;
			Iterator it = listaRecords.iterator();
			while(it.hasNext()) {
				AbstractRecord record = (AbstractRecord) it.next();
				BigInteger litri = getLitriValue(record);
				boolean boolLitri = !(litri == BigInteger.valueOf(0));
				
				if(record.getTipoRecord().equals(tipoRecord) && boolLitri) {
					counter++;
				}
			}
			return BigInteger.valueOf(counter);
		}
		
	
	/*
	protected void updateRecordNumbers() {
		
		frontespizio.setTotaleFileRecord(BigInteger.valueOf(listaRecord.size() + 1));
		frontespizio.setTotaleRecordQuadroA(QuadroUtil.countRecord(listaRecord, BigInteger.valueOf(10)));
		frontespizio.setTotaleRecordQuadroC(QuadroUtil.countRecord(listaRecord, BigInteger.valueOf(30)));
		frontespizio.setFlagQuadroB(QuadroUtil.countRecord(listaRecord, BigInteger.valueOf(20)));
	}*/

	protected void updateRecordNumbers() {
		
		//frontespizio.setTotaleFileRecord(BigInteger.valueOf(listaRecord.size() + 1));
		
		BigInteger countA = countRecord(listaRecord, BigInteger.valueOf(10));
		BigInteger countB = countRecord(listaRecord, BigInteger.valueOf(20));
		BigInteger countC =  BigInteger.valueOf(0);		
		//BigInteger countDatiContabili = frontespizio.getNumeroTotaleDatiContabili();
		
		if (countB != BigInteger.valueOf(0)) {
			countC = countRecord(listaRecord, BigInteger.valueOf(30)); 
		} 
		
		BigInteger countTotale = countA.add(countB).add(countC).add(BigInteger.valueOf(1));		
		
		frontespizio.setTotaleRecordQuadroA(countA);
		frontespizio.setFlagQuadroB(countB);
		frontespizio.setTotaleRecordQuadroC(countC);
		frontespizio.setTotaleFileRecord(countTotale);
		
	}

	public Object clone() throws CloneNotSupportedException {
		ProspettoConsumo clonato = new ProspettoConsumo();
	
		clonato.setResourceAccess(this.resourceAccess);
		clonato.setFrontespizio(this.frontespizio);
		clonato.setListaRecord(this.listaRecord);
	
		return clonato;
	}
	
	public Frontespizio getFrontespizio() {
		return frontespizio;
	}

	/**
	 * @param frontespizio
	 */
	public void setFrontespizio(Frontespizio frontespizio) {
		this.frontespizio = frontespizio;
	}


	public List getListaQuadro(int[] tipologieQuadro) {
		return QuadroUtil.getListaQuadro(listaRecord, tipologieQuadro);
	}
	
	// aggiunto reply
	// se tipoQuadroA  null agisce come getListaQuadro(int[] tipologieQuadro)
	public List getListaQuadro(int[] tipologieQuadro, String tipoQuadroA) {
		
		if( tipoQuadroA == null)		
			return QuadroUtil.getListaQuadro(listaRecord, tipologieQuadro);
		else
			return getListaQuadroA(tipoQuadroA);
			
	}
	
	// aggiunto reply
	// prende tutti i record del quadro a del tipo specificato
	public List getListaQuadroA( String tipoQuadroA ){
		
		List listaQuadriA= QuadroUtil.getListaQuadro(listaRecord, Quadro.QUADRO_A );// ha lo stesso valore di A2
				
				
				
		List listaRecordDaRimuovere = new LinkedList();

		for( int i=0; i < listaQuadriA.size() ; i++   ){
			
			AutotrasportatoriQuadroA quadroArecord = (AutotrasportatoriQuadroA)listaQuadriA.get(i);
			if( !quadroArecord.getTipoQuadroA().equals(tipoQuadroA) )
				listaRecordDaRimuovere.add( quadroArecord );
		}
		
		listaQuadriA.removeAll(listaRecordDaRimuovere);
		
		return listaQuadriA;
	}

	public void setListaQuadro(List listaQuadro, int[] tipologieQuadro) {
		QuadroUtil.removeListaQuadro(listaRecord, tipologieQuadro);
		QuadroUtil.addListaQuadro(listaRecord, listaQuadro);
	}
	
	// aggiunto reply
	public void setListaQuadroA(List listaQuadro, String tipologiaQuadroA ) {
		
		QuadroAutotrasportatoriUtil.removeListaQuadroA( listaRecord, tipologiaQuadroA );
		QuadroUtil.addListaQuadro( listaRecord, listaQuadro );
		
	}

	/**
	 * @return
	 */
	public ResourceAccess getResourceAccess() {
		return resourceAccess;
	}

	/**
	 * @param access
	 */
	public void setResourceAccess(ResourceAccess access) {
		resourceAccess = access;
	}

	public Object scriviOggetto(OutputStream out, int size) throws IOException {
		throw new UnsupportedOperationException();
	}

	public void setListaRecord(List listaRec){
		this.listaRecord=listaRec;
	}
	public List getListaRecord(){
		return this.listaRecord;
	}

	public Object scriviOggettoInterchange() throws IOException {
		throw new UnsupportedOperationException();
	}

	/**
	 * @return
	 */
	public List getListaDatiContabili() {
		return listaDatiContabili;
	}

	/**
	 * @param list
	 */
	public void setListaDatiContabili(List list) {
		listaDatiContabili = list;
	}

}
