package it.finanze.dogane.accise.aautotrasportatori.model;

import it.finanze.dogane.accise.aautotrasportatori.ui.AutotrasportatoriFrame;
import it.finanze.dogane.accise.aautotrasportatori.ui.quadri.Quadro;
import it.finanze.dogane.accise.model.AbstractRecord;

import java.io.BufferedReader;
import java.io.ByteArrayInputStream;
import java.io.IOException;

public class RecordFactory {

	public static final String FRONTESPIZIO_TYPE = "00";
	public static final String DATI_CONTABILI_TYPE = "01";
	public static final String QUADRO_A = "10";
	public static final String QUADRO_B = "20";
	public static final String QUADRO_C = "30";
	
	/**
	 * Ritorna un oggetto di tipo ElettricitaRecord oppure null se il file  stato letto completamente.
	 * @param is
	 * @return
	 * @throws IOException
	 */
	public static AutotrasportatoriRecord readNextRecord(BufferedReader reader)
		throws IOException {

		String record = reader.readLine();
		// Questa istruzione non funzioner se vferr modificato il terminatore di file
		if (record == null) {
			return null;
		}
		
		if( !checkRecordCompatibility(record) )
			return new AutotrasportatoriInvalidRecord();
			
		record += AbstractRecord.FINERECORD;

		ByteArrayInputStream recordIS =
			new ByteArrayInputStream(record.getBytes());

			AutotrasportatoriRecord autotrasportatoriRecord = null;

		String tipoRecord = record.substring(24,26);
		// 24 e la posizione di una stringa di 2 caratteri che indica il tipo record

		if (tipoRecord.equals(FRONTESPIZIO_TYPE)) {
			autotrasportatoriRecord = new Frontespizio();
		}else if (tipoRecord.equals(DATI_CONTABILI_TYPE)){			
			autotrasportatoriRecord = new DatiContabiliRecord();
		} else if (tipoRecord.equals(QUADRO_A)){				
			autotrasportatoriRecord = new AutotrasportatoriQuadroA();
		}else if (tipoRecord.equals(QUADRO_B)){
						
			 autotrasportatoriRecord = new AutotrasportatoriQuadroB();
		}else if (tipoRecord.equals(QUADRO_C)){
			
			autotrasportatoriRecord = new AutotrasportatoriQuadroC();


		} else{
			throw new RuntimeException("Tipo Record non riconosciuto. Record: " + record);
		}
		
	     
		autotrasportatoriRecord.leggiOggetto(recordIS);
		recordIS.close();

		return autotrasportatoriRecord;
	}
	
	// aggiunto reply
	// vede se i record in un formato compatibile 2006
	private static boolean checkRecordCompatibility( String record ){
		
		/*String tipoRecord = record.substring(24,26);
		
		if( tipoRecord.equals(QUADRO_A) ){
			
			for (int i = 0; i < AutotrasportatoriFrame.getNumeroAliquote(); i++) {
				
				if( record.charAt( record.length() -1) != Quadro.getTipoQuadroA(i).toCharArray()[0])				
					return false;
			}	
			
			
		}*/
		
		
		return true;
	
	}

}
