package it.finanze.dogane.accise.combo;

import it.finanze.dogane.accise.util.StringUtil;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.math.BigInteger;
import java.net.URL;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

public class ComboDataSource {

	private static ComboDataSource instance = new ComboDataSource();

	private HashMap periodi = new HashMap();

	private List province = null;

	

	private HashMap comuni = new HashMap();

	private HashMap capitoli = new HashMap();

	private HashMap cessioni = new HashMap();

	private List uffici = null;

	private HashMap tipologiaEstrVend = new HashMap();

	private List mesi = null;

	private List stati = null;

	private List esercenti = null;

	private List attivita = null;

	private ComboDataSource() {
	}

	public static ComboDataSource getInstance() {
		return instance;
	}

	

	public List getProvince() {
		if (province == null) {
			province = initProvince();
		}
		return province;
	}

	

	public TipoProvincia getProvincia(String codice) {
		Iterator it = getProvince().iterator();
		while (it.hasNext()) {
			TipoProvincia tipoProvincia = (TipoProvincia) it.next();
			if (tipoProvincia.getCodice().equals(codice)) {
				return tipoProvincia;
			}
		}

		return null;
	}

	public List getTipologieEstrazioneVendita(String quadro) {

		List tipologie = (List) tipologiaEstrVend.get(quadro);
		if (tipologie == null) {
			tipologie = initTipologiaEstrazioneVendita(quadro);
		}
		return tipologie;
	}

	public TipoEstrazioneVendita getTipologiaEstrazioneVendita(
		String quadro,
		String codiceTipologia) {
		Iterator it = getTipologieEstrazioneVendita(quadro).iterator();
		while (it.hasNext()) {
			TipoEstrazioneVendita tipoEstrazioneVendita =
				(TipoEstrazioneVendita) it.next();
			if (tipoEstrazioneVendita.getCodice().equals(codiceTipologia)) {
				return tipoEstrazioneVendita;
			}
		}
		return null;
	}

	private List initTipologiaEstrazioneVendita(String quadro) {

		List retvalue = null;
		URL url = null;
		if (quadro.equals("B")) {
			url =
				getClass().getClassLoader().getResource(
					"it/finanze/dogane/accise/combo/values/tipologieEstrazione.txt");
		} else if (quadro.equals("D")) {
			url =
				getClass().getClassLoader().getResource(
					"it/finanze/dogane/accise/combo/values/tipologieVendita.txt");
		}

		if (url == null) {
			throw new RuntimeException("File delle tipologie non  trovato.");
		}

		InputStream is;
		try {
			is = url.openStream();

			BufferedReader reader =
				new BufferedReader(new InputStreamReader(is));

			String tipologia = reader.readLine();
			retvalue = new LinkedList();
			while (tipologia != null) {
				TipoEstrazioneVendita tipoEstrazioneVendita =
					new TipoEstrazioneVendita();
				tipoEstrazioneVendita.setDescrizioneEstesa(tipologia);

				retvalue.add(tipoEstrazioneVendita);

				tipologia = reader.readLine();
			}

			reader.close();
			is.close();
		} catch (IOException e) {
			e.printStackTrace();
			throw new RuntimeException(e.getMessage());
		}

		return retvalue;
	}

	private List initProvince() {

		List retvalue = null;
		URL url =
			getClass().getClassLoader().getResource(
				"it/finanze/dogane/accise/combo/values/province.txt");

		if (url == null) {
			throw new RuntimeException("File delle province non trovato.");
		}

		InputStream is;
		try {
			is = url.openStream();

			BufferedReader reader =
				new BufferedReader(new InputStreamReader(is));

			String provincia = reader.readLine();
			retvalue = new LinkedList();
			while (provincia != null) {
				provincia = provincia.trim();
				TipoProvincia tipoProvincia = new TipoProvincia();
				tipoProvincia.setDescrizioneEstesa(provincia);

				retvalue.add(tipoProvincia);

				provincia = reader.readLine();
			}

			reader.close();
			is.close();
		} catch (IOException e) {
			e.printStackTrace();
			throw new RuntimeException(e.getMessage());
		}

		return retvalue;
	}

	
	

	public List getComuni(String codiceProvincia) {
		List comuniXProvincia = (List) comuni.get(codiceProvincia);
		if (comuniXProvincia == null) {
			comuniXProvincia = initComuni(codiceProvincia);
		}
		return comuniXProvincia;
	}

	public TipoComune getComune(String codiceProvincia, String codiceComune) {
		Iterator it = getComuni(codiceProvincia).iterator();
		while (it.hasNext()) {
			TipoComune tipoComune = (TipoComune) it.next();
			if (tipoComune.getCodice().equals(codiceComune)) {
				return tipoComune;
			}
		}
		return null;
	}

	public List getAliquote() {

		List listaAliquote = new LinkedList();
		listaAliquote.add("0.01859");
		listaAliquote.add("0.0204");
		return listaAliquote;
	}

	public List getQualificheDichiarante() {

		List listaQualifiche = new LinkedList();
		listaQualifiche.add("Rappresentante legale");
		listaQualifiche.add("Rappresentante negoziale");
		listaQualifiche.add("Titolare");
		return listaQualifiche;
	}

	public TipoComune getComunePerDescrizione(
		String codiceProvincia,
		String descrizioneComune) {
		Iterator it = getComuni(codiceProvincia).iterator();
		while (it.hasNext()) {
			TipoComune tipoComune = (TipoComune) it.next();
			if (tipoComune.getDescrizione().equals(descrizioneComune)) {
				return tipoComune;
			}
		}

		return null;
	}

	private List initComuni(String codiceProvincia) {
		List retvalue = null;
		codiceProvincia = codiceProvincia.toUpperCase();
		if (StringUtil.nullify(codiceProvincia) == null) {
			return null;
		}
		URL url =
			getClass().getClassLoader().getResource(
				"it/finanze/dogane/accise/combo/values/comuni/"
					+ codiceProvincia
					+ ".TXT");
					
					
	/* System.out.println("it/finanze/dogane/accise/combo/values/comuni/"+ codiceProvincia	+ ".TXT");
	 System.out.println(codiceProvincia);				
	 System.out.println(url);*/

		if (url == null) {
			throw new RuntimeException(
				"File dei comuni non trovato. Codice Provincia: "
					+ codiceProvincia);
		}

		InputStream is;
		try {
			is = url.openStream();

			BufferedReader reader =
				new BufferedReader(new InputStreamReader(is));

			String comune = reader.readLine();
			retvalue = new LinkedList();
			while (comune != null) {
				comune = comune.trim();
				TipoComune tipoComune = new TipoComune();
				tipoComune.setDescrizioneEstesa(comune);

				retvalue.add(tipoComune);

				comune = reader.readLine();
			}

			reader.close();
			is.close();
		} catch (IOException e) {
			e.printStackTrace();
			throw new RuntimeException(e.getMessage());
		}

		comuni.put(codiceProvincia, retvalue);

		return retvalue;
	}

	/**
	 * Ritorna la lista dei capitoli per quadro in input ci si aspetta la lettera identificativa del quadro.
	 * NB. Sebbene questo metodo sia generico per tutte le applicazioni i file da cui si caricano i dati
	 * appartengono ad ogni singolo progetto.
	 * @param quadro
	 * @return
	 */
	public List getCapitoli(String quadro) {
		List capitoliPerQuadro = (List) capitoli.get(quadro);
		if (capitoliPerQuadro == null) {
			capitoliPerQuadro = initCapitoli(quadro);
		}
		return capitoliPerQuadro;
	}
	public List getCapitoliArticolo(String quadro) {
		List capitoliPerQuadro = (List) capitoli.get(quadro);
		if (capitoliPerQuadro == null) {
			capitoliPerQuadro = initCapitoliArticolo(quadro);
		}
		return capitoliPerQuadro;
	}

	public TipoCapitolo getCapitolo(String quadro, String codice) {
		Iterator it = getCapitoli(quadro).iterator();
		while (it.hasNext()) {
			TipoCapitolo tipoCapitolo = (TipoCapitolo) it.next();
			if (tipoCapitolo.getCodice().trim().equals(codice)) {
				return tipoCapitolo;
			}
		}

		return null;
	}

	public TipoCapitoloArticolo getCapitoloArticolo(
		String quadro,
		String codice) {
		Iterator it = getCapitoliArticolo(quadro).iterator();
		while (it.hasNext()) {
			TipoCapitoloArticolo tipoCapitolo =
				(TipoCapitoloArticolo) it.next();
			if (tipoCapitolo.getCodice().trim().equals(codice)) {
				return tipoCapitolo;
			}
		}

		return null;
	}

	//inizializza Capitoli
	private List initCapitoli(String quadro) {
		List retvalue = null;
		quadro = quadro.toUpperCase();
		URL url =
			getClass().getClassLoader().getResource(
				"it/finanze/dogane/accise/combo/values/" + quadro + ".txt");

		if (url == null) {
			throw new RuntimeException("File dei capitoli non trovato.");
		}

		InputStream is;
		try {
			is = url.openStream();

			BufferedReader reader =
				new BufferedReader(new InputStreamReader(is));

			String capitolo = reader.readLine();
			retvalue = new LinkedList();
			while (capitolo != null) {
				TipoCapitolo tipoCapitolo = new TipoCapitolo();
				tipoCapitolo.setDescrizioneEstesa(capitolo);

				retvalue.add(tipoCapitolo);

				capitolo = reader.readLine();
			}

			reader.close();
			is.close();
		} catch (IOException e) {
			e.printStackTrace();
			throw new RuntimeException(e.getMessage());
		}

		capitoli.put(quadro, retvalue);

		return retvalue;
	}

	private List initCapitoliArticolo(String quadro) {
		List retvalue = null;
		quadro = quadro.toUpperCase();
		URL url =
			getClass().getClassLoader().getResource(
				"it/finanze/dogane/accise/combo/values/" + quadro + ".txt");

		if (url == null) {
			throw new RuntimeException("File dei capitoli non trovato.");
		}

		InputStream is;
		try {
			is = url.openStream();

			BufferedReader reader =
				new BufferedReader(new InputStreamReader(is));

			String capitolo = reader.readLine();
			retvalue = new LinkedList();
			while (capitolo != null) {
				TipoCapitoloArticolo tipoCapitolo = new TipoCapitoloArticolo();
				tipoCapitolo.setDescrizioneEstesa(capitolo);

				retvalue.add(tipoCapitolo);

				capitolo = reader.readLine();
			}

			reader.close();
			is.close();
		} catch (IOException e) {
			e.printStackTrace();
			throw new RuntimeException(e.getMessage());
		}

		capitoli.put(quadro, retvalue);

		return retvalue;
	}

	public List getMesi() {
		if (mesi == null) {
			mesi = initMesi();
		}
		return mesi;
	}

	public TipoMese getMese(String codice) {
		Iterator it = getMesi().iterator();
		while (it.hasNext()) {
			TipoMese tipoMese = (TipoMese) it.next();
			if (tipoMese.getCodice().equals(codice)) {
				return tipoMese;
			}
		}

		return null;
	}

	private List initMesi() {

		List retvalue = null;
		URL url =
			getClass().getClassLoader().getResource(
				"it/finanze/dogane/accise/combo/values/mesi.txt");

		if (url == null) {
			throw new RuntimeException("File dei mesi non trovato.");
		}

		InputStream is;
		try {
			is = url.openStream();

			BufferedReader reader =
				new BufferedReader(new InputStreamReader(is));

			String mese = reader.readLine();
			retvalue = new LinkedList();
			while (mese != null) {
				TipoMese tipoMese = new TipoMese();
				tipoMese.setDescrizioneEstesa(mese);

				retvalue.add(tipoMese);

				mese = reader.readLine();
			}

			reader.close();
			is.close();
		} catch (IOException e) {
			e.printStackTrace();
			throw new RuntimeException(e.getMessage());
		}

		return retvalue;
	}

	/**
	 * Ritorna la lista delle cessioni per quadro in input ci si aspetta la lettera identificativa del quadro.
	 * NB. Sebbene questo metodo sia generico per tutte le applicazioni i file da cui si caricano i dati
	 * appartengono ad ogni singolo progetto.
	 * @param quadro
	 * @return
	 */
	public List getCessioni(String quadro) {
		List cessioniPerQuadro = (List) cessioni.get(quadro);
		if (cessioniPerQuadro == null) {
			cessioniPerQuadro = initCessioni(quadro);
		}
		return cessioniPerQuadro;
	}

	public TipoCessione getCessione(String quadro, String codice) {
		Iterator it = getCessioni(quadro).iterator();
		while (it.hasNext()) {
			TipoCessione tipoCessione = (TipoCessione) it.next();
			if (tipoCessione.getCodice().equals(codice)) {
				return tipoCessione;
			}
		}

		return null;
	}

	//inizializza Cessioni
	private List initCessioni(String quadro) {
		List retvalue = null;
		quadro = quadro.toUpperCase();
		URL url =
			getClass().getClassLoader().getResource(
				"it/finanze/dogane/accise/eelettrica/combo/values/cessioni/"
					+ quadro
					+ ".txt");

		if (url == null) {
			throw new RuntimeException("File delle cessioni non trovato.");
		}

		InputStream is;
		try {
			is = url.openStream();

			BufferedReader reader =
				new BufferedReader(new InputStreamReader(is));

			String cessione = reader.readLine();
			retvalue = new LinkedList();
			while (cessione != null) {
				TipoCessione tipoCessione = new TipoCessione();
				tipoCessione.setDescrizioneEstesa(cessione);

				retvalue.add(tipoCessione);

				cessione = reader.readLine();
			}

			reader.close();
			is.close();
		} catch (IOException e) {
			e.printStackTrace();
			throw new RuntimeException(e.getMessage());
		}

		cessioni.put(quadro, retvalue);

		return retvalue;
	}

	private List initUffici() {

		List retvalue = null;
		URL url =
			getClass().getClassLoader().getResource(
				"it/finanze/dogane/accise/combo/values/tipologiaUffici.txt");

		if (url == null) {
			throw new RuntimeException("File delle province non trovato.");
		}

		InputStream is;
		try {
			is = url.openStream();

			BufferedReader reader =
				new BufferedReader(new InputStreamReader(is));

			String ufficio = reader.readLine();
			retvalue = new LinkedList();
			while (ufficio != null) {
				TipoUfficio tipoUfficio = new TipoUfficio();
				tipoUfficio.setDescrizioneEstesa(ufficio);

				retvalue.add(tipoUfficio);

				ufficio = reader.readLine();
			}

			reader.close();
			is.close();
		} catch (IOException e) {
			e.printStackTrace();
			throw new RuntimeException(e.getMessage());
		}

		return retvalue;
	}

	public List getUffici() {
		if (uffici == null) {
			uffici = initUffici();
		}
		return uffici;
	}

	public TipoUfficio getUfficio(String codice) {
		Iterator it = getUffici().iterator();
		while (it.hasNext()) {
			TipoUfficio tipoUfficio = (TipoUfficio) it.next();
			if (tipoUfficio.getCodice().equals(codice)) {
				return tipoUfficio;
			}
		}

		return null;
	}

	private List initPeriodi(BigInteger anno) {

		List retvalue = null;
		URL url =
			getClass().getClassLoader().getResource(
				"it/finanze/dogane/accise/combo/values/periodi/" + anno + ".txt");

		if (url == null) {
			throw new RuntimeException("File dei periodi non trovato per anno: " + anno);
		}

		InputStream is;
		try {
			is = url.openStream();

			BufferedReader reader =
				new BufferedReader(new InputStreamReader(is));

			String periodo = reader.readLine();
			retvalue = new LinkedList();
			while (periodo != null) {
				TipoPeriodo tipoPeriodo = new TipoPeriodo();
				tipoPeriodo.setDescrizioneEstesa(periodo);

				retvalue.add(tipoPeriodo);

				periodo = reader.readLine();
			}

			reader.close();
			is.close();
		} catch (IOException e) {
			e.printStackTrace();
			throw new RuntimeException(e.getMessage());
		}

		periodi.put(anno, retvalue);

		return retvalue;
	}

	public List getPeriodi(BigInteger anno) {
		List retvalue = (List)periodi.get(anno);
		if (retvalue == null) {
			retvalue = initPeriodi(anno);
		}
		return retvalue;
	}

	public TipoPeriodo getPeriodo(BigInteger anno, BigInteger codice) {
		Iterator it = getPeriodi(anno).iterator();
		while (it.hasNext()) {
			TipoPeriodo tipoPeriodo = (TipoPeriodo) it.next();
			String itemCodice = tipoPeriodo.getCodice();
			String strCodice = String.valueOf(codice);
			if(strCodice.length() == 1) {
				itemCodice = tipoPeriodo.getCodicePeriodo();
			}
			if (itemCodice.equals(strCodice)) {
				return tipoPeriodo;
			}
		}

		return null;
	}

	public TipoPeriodo getPeriodoPerDescrizione(BigInteger anno, String descrizionePeriodo) {
		Iterator it = getPeriodi(anno).iterator();
		while (it.hasNext()) {
			TipoPeriodo tipoPeriodo = (TipoPeriodo) it.next();
			if (tipoPeriodo.getDescrizione().equals(descrizionePeriodo)) {
				return tipoPeriodo;
			}
		}

		return null;
	}

	private List initStati() {

		List retvalue = null;
		URL url =
			getClass().getClassLoader().getResource(
				"it/finanze/dogane/accise/combo/values/stati.txt");

		if (url == null) {
			throw new RuntimeException("File degli stati non trovato.");
		}

		InputStream is;
		try {
			is = url.openStream();

			BufferedReader reader =
				new BufferedReader(new InputStreamReader(is));

			String stato = reader.readLine();
			retvalue = new LinkedList();
			while (stato != null) {
				TipoStato tipoStato = new TipoStato();
				tipoStato.setDescrizioneEstesa(stato);

				retvalue.add(tipoStato);

				stato = reader.readLine();
			}

			reader.close();
			is.close();
		} catch (IOException e) {
			e.printStackTrace();
			throw new RuntimeException(e.getMessage());
		}

		return retvalue;
	}

	public List getStati() {
		if (stati == null) {
			stati = initStati();
		}
		return stati;
	}

	public TipoStato getStato(String codice) {
		Iterator it = getStati().iterator();
		while (it.hasNext()) {
			TipoStato tipoStato = (TipoStato) it.next();
			if (tipoStato.getCodice().equals(codice)) {
				return tipoStato;
			}
		}

		return null;
	}

	private List initEsercenti() {

		List retvalue = null;
		URL url =
			getClass().getClassLoader().getResource(
				"it/finanze/dogane/accise/combo/values/tipologiaEsercente.txt");

		if (url == null) {
			throw new RuntimeException("File degli esercenti non trovato.");
		}

		InputStream is;
		try {
			is = url.openStream();

			BufferedReader reader =
				new BufferedReader(new InputStreamReader(is));

			String esercente = reader.readLine();
			retvalue = new LinkedList();
			while (esercente != null) {
				TipoEsercente tipoEsercente = new TipoEsercente();
				tipoEsercente.setDescrizioneEstesa(esercente);

				retvalue.add(tipoEsercente);

				esercente = reader.readLine();
			}

			reader.close();
			is.close();
		} catch (IOException e) {
			e.printStackTrace();
			throw new RuntimeException(e.getMessage());
		}

		return retvalue;
	}

	public List getEsercenti() {
		if (esercenti == null) {
			esercenti = initEsercenti();
		}
		return esercenti;
	}

	public TipoEsercente getEsercente(BigInteger codice) {
		Iterator it = getEsercenti().iterator();
		while (it.hasNext()) {
			TipoEsercente tipoEsercente = (TipoEsercente) it.next();
			if (tipoEsercente.getCodice().equals(String.valueOf(codice))) {
				return tipoEsercente;
			}
		}

		return null;
	}
	private List initAttivita() {

		List retvalue = null;
		URL url =
			getClass().getClassLoader().getResource(
				"it/finanze/dogane/accise/combo/values/codiciAttivita.txt");

		if (url == null) {
			throw new RuntimeException("File delle attivit non trovato.");
		}

		InputStream is;
		try {
			is = url.openStream();

			BufferedReader reader =
				new BufferedReader(new InputStreamReader(is));

			String attivita = reader.readLine();
			retvalue = new LinkedList();
			while (attivita != null) {
				TipoAttivita tipoAttivita = new TipoAttivita();
				tipoAttivita.setDescrizioneEstesa(attivita);

				retvalue.add(tipoAttivita);

				attivita = reader.readLine();
			}

			reader.close();
			is.close();
		} catch (IOException e) {
			e.printStackTrace();
			throw new RuntimeException(e.getMessage());
		}

		return retvalue;
	}

	public List getAttivitaLista() {
		if (attivita == null) {
			attivita = initAttivita();
		}
		return attivita;
	}

	public TipoAttivita getAttivita(String codice) {
		Iterator it = getAttivitaLista().iterator();
		while (it.hasNext()) {
			TipoAttivita tipoAttivita = (TipoAttivita) it.next();
			if (tipoAttivita.getCodice().equals(String.valueOf(codice))) {
				return tipoAttivita;
			}
		}

		return null;
	}

}
