package it.finanze.dogane.accise.pdf;

import it.finanze.dogane.accise.ui.PdfFileFilter;
import it.finanze.dogane.accise.ui.pedal.PdfDecoder;

import java.awt.Component;
import java.awt.print.PageFormat;
import java.awt.print.Pageable;
import java.awt.print.PrinterException;
import java.awt.print.PrinterJob;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;

import javax.jnlp.FileContents;
import javax.jnlp.FileOpenService;
import javax.jnlp.FileSaveService;
import javax.jnlp.PrintService;
import javax.jnlp.ServiceManager;
import javax.jnlp.UnavailableServiceException;
import javax.print.attribute.HashPrintRequestAttributeSet;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;

import org.jpedal.exception.PdfException;

public class PdfResourceAccess {

	private static FileOpenService fileOpenService = null;
	private static FileSaveService fileSaveService = null;
	private static PrintService printService = null;

	private static FileContents contents = null;
	private static File file = null;

	private static String[] extensions = {"pdf", "PDF"};


	static {
		try {
			fileOpenService = (FileOpenService)ServiceManager.lookup("javax.jnlp.FileOpenService");
			fileSaveService = (FileSaveService)ServiceManager.lookup("javax.jnlp.FileSaveService");
			printService = (PrintService)ServiceManager.lookup("javax.jnlp.PrintService");
		} catch(UnavailableServiceException e) {
			fileOpenService = null;
			fileSaveService = null;
			printService = null;
		}
	}

	public static boolean saveAsFile(Component component, byte[] document) throws IOException {
		boolean retvalue = false;
		if(fileOpenService != null) {
			retvalue = saveAsFileFromService(component, document); // Se si lancia attraverso Java Web Start
		} else {
			retvalue = saveAsFileStandardWay(component, document); // Se non si lancia attraverso Java Web Start
		}
		return retvalue;
	}

	/**
	 * @param component
	 * @param prospect
	 */
	private static boolean saveAsFileStandardWay(Component component, byte[] document) throws IOException {
		boolean retvalue = false;
		JFileChooser fileChooser = new JFileChooser();
		fileChooser.setFileFilter(new PdfFileFilter());
		
		if(file != null) {
			fileChooser.setSelectedFile(file);
		}
		
		int userAction = fileChooser.showSaveDialog(component);

		if (userAction == JFileChooser.APPROVE_OPTION) {
			
			file = fileChooser.getSelectedFile();
			
			/**************/
			String fileExtension = ".pdf";
			//se l'estensione non  .pdf, l'aggiungiamo noi
			if(!file.getName().endsWith(fileExtension)){
				
				String strPath = file.getAbsolutePath();
				file = new File(strPath+fileExtension);				
			}
						
			//se il file gi esiste
			if(file.exists()){
								
				int confirm = JOptionPane.showOptionDialog(null,
					"Il file "+file.getName()+" gi esiste. Sovrascriverlo ?", "Sovrascrivere File",
					JOptionPane.YES_NO_OPTION,JOptionPane.QUESTION_MESSAGE,null, null, null);
					if (confirm == 1)//No				
						return false;					
			}
			/**************/
			
			
			
			FileOutputStream os = new FileOutputStream(file);
			os.write(document);
			os.close();
			retvalue = true;
		}
		return retvalue;
	}

	/**
	 * @param component
	 * @param prospect
	 */
	private static boolean saveAsFileFromService(Component component, byte[] document) throws IOException {
		boolean retvalue = false;
		
		ByteArrayInputStream is = new ByteArrayInputStream(document);
		
		String suggestedName = "Prospetto.pdf";
		if(contents != null) {
			suggestedName = contents.getName();
		}
		
		FileContents newContents = fileSaveService.saveFileDialog(null, extensions, is, suggestedName);
		if(newContents != null) {
			contents = newContents;
			retvalue = true; 
		}

		return retvalue;
	}

	public static void print(PdfDecoder decoder) throws Exception {
		if(printService != null) {
			printFromService(decoder); // Se si lancia attraverso Java Web Start
		} else {
			printStandardWay(decoder); // Se non si lancia attraverso Java Web Start
		}
		
	}

	private static void printFromService(PdfDecoder decoder) throws Exception {
		PageFormat pageformat = printService.getDefaultPage();
		decoder.setPrintAutoRotateAndCenter(true);
		decoder.setPageFormat(pageformat);
		decoder.setPagePrintRange(1, decoder.getPageCount());
		
		printService.print((Pageable)decoder);
	}

	public static void printStandardWay(PdfDecoder decoder) throws Exception {
		PrinterJob printerjob = PrinterJob.getPrinterJob();
		PageFormat pageformat = printerjob.defaultPage();
		decoder.setPrintAutoRotateAndCenter(true);		
		decoder.setPageFormat(pageformat);
		
		decoder.setPagePrintRange(1, decoder.getPageCount());
		
		//decoder.disableBorderForPrinting();
		printerjob.setPageable(decoder);
		if(printerjob.printDialog()) {
			printerjob.print();
		}
	}

}
