package it.finanze.dogane.accise.ui;

import it.finanze.dogane.accise.pdf.PdfResourceAccess;
import it.finanze.dogane.accise.ui.actions.BaseAbstractAction;
import it.finanze.dogane.accise.ui.actions.CloseAction;
import it.finanze.dogane.accise.ui.actions.ExitAction;
import it.finanze.dogane.accise.ui.actions.IngrandisciAction;
import it.finanze.dogane.accise.ui.actions.NextAction;
import it.finanze.dogane.accise.ui.actions.PreviousAction;
import it.finanze.dogane.accise.ui.actions.PrintPdfAction;
import it.finanze.dogane.accise.ui.actions.RiduciAction;
import it.finanze.dogane.accise.ui.actions.SavePdfAction;
import it.finanze.dogane.accise.ui.pedal.*;

import java.awt.Color;
import java.awt.Frame;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.IOException;

import javax.swing.JDialog;
import javax.swing.JOptionPane;


public class PdfViewer extends JDialog implements ActionListener {

	private javax.swing.JPanel jContentPane = null;

	private javax.swing.JToolBar jToolBar = null;

	private SavePdfAction savePdfAction = null;
	private PreviousAction previousAction = null;
	private NextAction nextAction = null;
	private CloseAction closeAction = null;
	
	private PrintPdfAction printPdfAction = null;
	private IngrandisciAction ingrandisciAction = null;
	private RiduciAction riduciAction = null;

	private javax.swing.JScrollPane jScrollPane = null;
	
	/**
	 * Versione modificata di PdfDecoder
	 */
	private PdfDecoder decoder = null;
	
	private byte[] pdfDocumentContent = null;
	private Frame frame = null;
	
	private int currentPage = 1;
	private float magnify = 1.0f;
	private int pageCount = 0;

	private javax.swing.JPanel jPanelInfo = null;
	private javax.swing.JLabel jLabelPageCounter = null;
	/**
	 * This is the default constructor
	 */
	public PdfViewer(Frame frame, byte[] pdf) {
		super(frame);
		initialize();
		
		this.frame = frame;
		this.pdfDocumentContent = pdf;
		
		decoder = new PdfDecoder();
		
		try {
			decoder.openPdfArray(pdf);
			decoder.decodePage(currentPage);
			decoder.setPageParameters(magnify, currentPage);
//			decoder.setBorder(javax.swing.BorderFactory.createLineBorder(java.awt.Color.gray,2));
			decoder.setBackground(Color.LIGHT_GRAY);

			previousAction.setEnabled(false);
			pageCount = decoder.getPageCount();
			if(pageCount == 1) {
				nextAction.setEnabled(false);
			}
			
			jScrollPane.setViewportView(decoder);
			
			updateCurrentPageNumber();						
		} catch (Exception e) {
			e.printStackTrace();
		}
		
	}

	/**
	 * This method initializes this
	 * 
	 * @return void
	 */
	private void initialize() {
		// this.setSize(800, 600);
		this.setSize(RealDim.getAbsoluteXValue(800), RealDim.getAbsoluteYValue(600));
		initActions();
		this.setContentPane(getJContentPane());
		this.setModal(true);
		this.setDefaultCloseOperation(javax.swing.WindowConstants.DO_NOTHING_ON_CLOSE);
		this.setTitle("Visualizzatore di PDF");
	}

	private void initActions() {

		savePdfAction = new SavePdfAction();
		registerAction(savePdfAction);

		previousAction = new PreviousAction();
		registerAction(previousAction);

		nextAction = new NextAction();
		registerAction(nextAction);

		closeAction = new CloseAction();
		registerAction(closeAction);

		printPdfAction = new PrintPdfAction();
		registerAction(printPdfAction);

		ingrandisciAction = new IngrandisciAction();
		registerAction(ingrandisciAction);

		riduciAction = new RiduciAction();
		registerAction(riduciAction);
	}

	private void registerAction(BaseAbstractAction action) {
		action.addActionListener(this);
	}

	private void updateCurrentPageNumber() {
		jLabelPageCounter.setText("Pagina corrente " + currentPage + " di " + pageCount + " Pagine");
	}

	public void actionPerformed(ActionEvent event) {
		try {
			if (savePdfAction.getActionCommand().equals(event.getActionCommand())) {
				doSave();
			} else if (previousAction.getActionCommand().equals(event.getActionCommand())) {
				doPrevious();
			} else if (nextAction.getActionCommand().equals(event.getActionCommand())) {
				doNext();
			} else if (closeAction.getActionCommand().equals(event.getActionCommand())) {
				this.setVisible(false);
			} else if (printPdfAction.getActionCommand().equals(event.getActionCommand())) {
				doPrint();
			} else if (ingrandisciAction.getActionCommand().equals(event.getActionCommand())) {
				doIngrandisci();
			} else if (riduciAction.getActionCommand().equals(event.getActionCommand())) {
				doRiduci();
			}
		}
		catch(Exception e) {
			e.printStackTrace();
			JOptionPane.showMessageDialog(
				this,
				"Errore durante l'operazione richiesta: \n" + e.getMessage(),
				"Errore",
				JOptionPane.ERROR_MESSAGE);
		}
	}

	private void doSave() throws IOException {
		PdfResourceAccess.saveAsFile(frame, pdfDocumentContent);
	}

	private void doPrevious() throws Exception {
		if(currentPage > 1) {
			nextAction.setEnabled(true);
			decoder.decodePage(currentPage-1);
			decoder.setPageParameters(magnify, currentPage-1);
			currentPage--;
			if(currentPage == 1) {
				previousAction.setEnabled(false);
			}

			decoder.invalidate();
			jScrollPane.repaint();
			updateCurrentPageNumber();
		}
	}

	private void doNext() throws Exception {
		if(currentPage < pageCount) {
			previousAction.setEnabled(true);
			decoder.decodePage(currentPage+1);
			decoder.setPageParameters(magnify, currentPage+1);
			currentPage++;
			if(currentPage == pageCount) {
				nextAction.setEnabled(false);
			}
			
			decoder.invalidate();
			jScrollPane.repaint();
			updateCurrentPageNumber();
		}
	}

	private void doIngrandisci() throws Exception {
		magnify += 0.1;

		decoder.decodePage(currentPage);
		decoder.setPageParameters(magnify, currentPage);
		decoder.invalidate();
		jScrollPane.setViewportView(decoder);
		jScrollPane.repaint();
	}

	private void doRiduci() throws Exception {
		magnify -= 0.1;
		
		decoder.decodePage(currentPage);
		decoder.setPageParameters(magnify, currentPage);
		decoder.invalidate();
		jScrollPane.setViewportView(decoder);
		jScrollPane.repaint();
	}

	private void doPrint() throws Exception {
//		throw new RuntimeException("Funzionalit non ancora implementata");

		PdfResourceAccess.print(decoder);

	}

	/**
	 * This method initializes jContentPane
	 * 
	 * @return javax.swing.JPanel
	 */
	private javax.swing.JPanel getJContentPane() {
		if (jContentPane == null) {
			jContentPane = new javax.swing.JPanel();
			jContentPane.setLayout(new java.awt.BorderLayout());
			jContentPane.add(getJToolBar(), java.awt.BorderLayout.NORTH);
			jContentPane.add(getJScrollPane(), java.awt.BorderLayout.CENTER);
			jContentPane.add(getJPanelInfo(), java.awt.BorderLayout.SOUTH);
		}
		return jContentPane;
	}
	/**
	 * This method initializes jToolBar
	 * 
	 * @return javax.swing.JToolBar
	 */
	private javax.swing.JToolBar getJToolBar() {
		if(jToolBar == null) {
			jToolBar = new javax.swing.JToolBar();

			jToolBar.add(savePdfAction);
			jToolBar.add(printPdfAction);
			jToolBar.addSeparator();
			jToolBar.add(previousAction);
			jToolBar.add(nextAction);
			jToolBar.addSeparator();
			jToolBar.add(ingrandisciAction);
			jToolBar.add(riduciAction);
			jToolBar.addSeparator();
			jToolBar.add(closeAction);
			
		}
		return jToolBar;
	}
	/**
	 * This method initializes jScrollPane
	 * 
	 * @return javax.swing.JScrollPane
	 */
	private javax.swing.JScrollPane getJScrollPane() {
		if(jScrollPane == null) {
			jScrollPane = new javax.swing.JScrollPane();
		}
		return jScrollPane;
	}
	/**
	 * This method initializes jPanelInfo
	 * 
	 * @return javax.swing.JPanel
	 */
	private javax.swing.JPanel getJPanelInfo() {
		if(jPanelInfo == null) {
			jPanelInfo = new javax.swing.JPanel();
			jPanelInfo.add(getJLabelPageCounter(), null);
		}
		return jPanelInfo;
	}
	/**
	 * This method initializes jLabelPageCounter
	 * 
	 * @return javax.swing.JLabel
	 */
	private javax.swing.JLabel getJLabelPageCounter() {
		if(jLabelPageCounter == null) {
			jLabelPageCounter = new javax.swing.JLabel();
			jLabelPageCounter.setText("Pagina di");
		}
		return jLabelPageCounter;
	}
}
