package it.finanze.dogane.accise.ui.laf;

import java.awt.Component;
import java.awt.Frame;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.lang.reflect.Method;

import javax.swing.ButtonGroup;
import javax.swing.JCheckBoxMenuItem;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JRadioButtonMenuItem;
import javax.swing.JRootPane;
import javax.swing.LookAndFeel;
import javax.swing.SwingUtilities;
import javax.swing.UIManager;
import javax.swing.UnsupportedLookAndFeelException;
import javax.swing.plaf.metal.DefaultMetalTheme;
import javax.swing.plaf.metal.MetalLookAndFeel;
import javax.swing.plaf.metal.MetalTheme;

/**
 * Look And Feel Manager
 */
public class LAFManager implements ActionListener {

	private static final String LAF_INFO_PROPERTY = "LAF_INFO_PROPERTY";
	private static LAFManager instance = new LAFManager();
	
	private JFrame window;
	private boolean decoration = false;
	
	private LAFManager() {
	}

	public static LAFManager getInstance() {
		return instance;
	}

	public void initLookMenu(JMenuBar menuBar, JFrame mainWin) {
		JMenu lookMenu = new JMenu("Aspetto");
		menuBar.add(lookMenu);
		
		window = mainWin;
		decoration = JFrame.isDefaultLookAndFeelDecorated();
		
		JCheckBoxMenuItem decore = new JCheckBoxMenuItem("Decorazione", decoration);
		decore.putClientProperty(LAFManager.LAF_INFO_PROPERTY, "Decore");
		decore.addActionListener(this);
		lookMenu.add(decore);
		lookMenu.addSeparator();

		// Menu for the look and feels (lafs).
		UIManager.LookAndFeelInfo[]        lafs = UIManager.getInstalledLookAndFeels();
		ButtonGroup                        lafGroup = new ButtonGroup();

		for(int i = 0; i < lafs.length; i++) {
			JRadioButtonMenuItem rb = new JRadioButtonMenuItem(lafs[i].getName());
			lookMenu.add(rb);
			rb.setSelected(UIManager.getLookAndFeel().getName().equals(lafs[i].getName()));
			rb.putClientProperty(LAFManager.LAF_INFO_PROPERTY, lafs[i]);
			rb.addActionListener(this);
			lafGroup.add(rb);
		}
		
		// SubMenu per i temi
		JMenu themeMenu = new JMenu("Tema");
		lookMenu.add(themeMenu);

		ButtonGroup themeGroup = new ButtonGroup();
		JRadioButtonMenuItem menuItem = null;
		
		// Inizio del codice con introspezione
		// Questo codice  utilizzato per evitare le differenze di compilazione dovute al jdk 1.4 - 1.5
		MetalTheme current = null;
		try {
			// Mi prendo il tema corrente --> current = MetalLookAndFeel.getCurrentTheme();
			Class clazz = MetalLookAndFeel.class;
			Method method = clazz.getMethod("getCurrentTheme", null);
			current = (MetalTheme)method.invoke(null , null);
			
		}
		catch(Exception e) {/*e.printStackTrace();*/}
		
		if(current != null) {
			menuItem = new JRadioButtonMenuItem(current.getName());
			menuItem.putClientProperty(LAFManager.LAF_INFO_PROPERTY, current);
			menuItem.addActionListener(this);
			themeMenu.add(menuItem);
			themeGroup.add(menuItem);
			menuItem.setSelected(true);
		}

		try {
			Class oceanClass = Class.forName("javax.swing.plaf.metal.OceanTheme");
			MetalTheme ocean = (MetalTheme)oceanClass.newInstance();
			if(current == null || !(ocean.getName().equals(current.getName()))) {
				// Poi il nuovo 1.5 Java (se non l'ho gi inserito e se esiste ovviamente...)
				menuItem = new JRadioButtonMenuItem(ocean.getName());
				menuItem.putClientProperty(LAFManager.LAF_INFO_PROPERTY, ocean);
				menuItem.addActionListener(this);
				themeMenu.add(menuItem);
				themeGroup.add(menuItem);		
			}

		}
		catch(Exception e) {/*e.printStackTrace();*/}
		// Fine del codice con introspezione

		MetalTheme classic = new DefaultMetalTheme();
		if(current == null || !(classic.getName().equals(current.getName()))) {
			// Poi il classico Java (se non l'ho gi inserito...)
			menuItem = new JRadioButtonMenuItem(classic.getName());
			menuItem.putClientProperty(LAFManager.LAF_INFO_PROPERTY, classic);
			menuItem.addActionListener(this);
			themeMenu.add(menuItem);
			themeGroup.add(menuItem);		
		}

		// Poi quelli previsti da questo progetto
		MetalTheme[] themes = LAFManager.getInstance().getThemes();
		if(themes != null) {
			for(int i = 0; i < themes.length; i++) {
				menuItem = new JRadioButtonMenuItem(themes[i].getName());
				menuItem.putClientProperty(LAFManager.LAF_INFO_PROPERTY, themes[i]);
				menuItem.addActionListener(this);
				themeMenu.add(menuItem);
				themeGroup.add(menuItem);		
			}
		}
	}

	private MetalTheme[] getThemes() {
		MetalTheme[] themes = new MetalTheme[2];
		
		themes[0] = new AquaTheme();
		themes[1] = new ContrastTheme();
		
		// TODO Sarebbe auspicabile avere anche dei temi impostabili da file di configurazione

		return themes;
	}

	public void actionPerformed(ActionEvent event) {
		if(event.getSource() instanceof JComponent) {
			JComponent source = (JComponent)event.getSource();
			
			Object property = source.getClientProperty(LAF_INFO_PROPERTY);
			if(property == null) {

				updateLook();

			} else if(property instanceof UIManager.LookAndFeelInfo) {

				UIManager.LookAndFeelInfo info = (UIManager.LookAndFeelInfo)property;
				updateLook(info);

			} else if(property instanceof MetalTheme) {

				MetalLookAndFeel.setCurrentTheme((MetalTheme)property);
				updateLook();

			} else if(property instanceof String) {
				String command = (String)property;
				if(command.equals("Decore")) {
					boolean d = swapDecoration();
					JCheckBoxMenuItem decore = (JCheckBoxMenuItem)source;
					decore.setSelected(d);
				}
			}
		}
	}
	
	/**
	 * Aggiorna il look and feel dei componenti registrati
	 */
	public void updateLook() {
		try {
			UIManager.setLookAndFeel(UIManager.getLookAndFeel());
		} catch (UnsupportedLookAndFeelException e) {
			e.printStackTrace();
		}

		forceLook(window);

		LookAndFeel laf = UIManager.getLookAndFeel();
		if(laf.getSupportsWindowDecorations()) {
			forceDecoration(decoration);
		} else {
			forceDecoration(false);
		}
	}

	/**
	 * Aggiorna il look and feel dei componenti registrati con l'info passata in input
	 */
	public void updateLook(UIManager.LookAndFeelInfo info) {
		try {
			UIManager.setLookAndFeel(info.getClassName());
				
			updateLook();
		}
		catch (Exception e) {
			e.printStackTrace();
		}
	}

	private void forceLook(Component c) {
		SwingUtilities.updateComponentTreeUI(c);
	}

	private void forceDecoration(boolean decorate) {
		JFrame tmp = new JFrame();
		tmp.setVisible(true); // Per assicurarsi che la macchina virtuale non si spenga.
			
		imp_forceDecoration(window, decorate);
	
		tmp.dispose(); // ora non serve pi
	}

	private void forceDecoration(Frame frame, boolean decorate) {
		JFrame tmp = new JFrame();
		tmp.setVisible(true); // Per assicurarsi che la macchina virtuale non si spenga.
			
		imp_forceDecoration(frame, decorate);
	
		tmp.dispose(); // ora non serve pi
	}

	private void imp_forceDecoration(Frame frame, boolean decorate) {
		boolean resetVisibility = false;
		resetVisibility = frame.isVisible();
		frame.dispose();
		
		frame.setUndecorated(decorate);
		if(frame instanceof JFrame) {
			JFrame jframe = (JFrame)frame;
			if(decorate) {
				jframe.getRootPane().setWindowDecorationStyle(JRootPane.FRAME); 
			} else {
				jframe.getRootPane().setWindowDecorationStyle(JRootPane.NONE); 
			}
		}
		
		frame.setVisible(resetVisibility);
	}

	/**
	 * Attiva o disattiva la decorazione se il Look and Feel ha questa possibilit
	 * Per testare se la decorazione  attiva o meno si pu controllare il valore ritornato da questo metodo.
	 * @return
	 */
	public boolean swapDecoration() {
		return updateDecoration(!decoration);
	}

	/**
	 * Riapplica la decorazione attiva se il Look and Feel ha questa possibilit
	 * Per testare se la decorazione  attiva o meno si pu controllare il valore ritornato da questo metodo.
	 * @return
	 */
	public boolean updateDecoration() {
		return updateDecoration(decoration);
	}

	/**
	 * Imposta la decorazione se il Look and Feel ha questa possibilit.
	 * Per testare se la decorazione  attiva o meno si pu controllare il valore ritornato da questo metodo.
	 * @param decorate
	 */
	public boolean updateDecoration(boolean decorate) {
		boolean retvalue = false;
		LookAndFeel laf = UIManager.getLookAndFeel();
		if(laf.getSupportsWindowDecorations()) {
			forceDecoration(decorate); // Decora la finestra principale
			JFrame.setDefaultLookAndFeelDecorated(decorate); // Imposta la decorazione per eventuali altre finestre
			JDialog.setDefaultLookAndFeelDecorated(decorate); // Imposta la decorazione per eventuali altre finestre di dialogo
			retvalue = decorate;
			decoration = decorate;
		
		}

		return retvalue;
	}
}
