package it.finanze.dogane.accise.ui.validation;

import javax.swing.text.AttributeSet;
import javax.swing.text.BadLocationException;
import javax.swing.text.PlainDocument;

/**
 * Utilizzato per la validazione dei JTextField che devono contenere solo dati alfanumerici 
 */
public class StringDocument extends PlainDocument {
	
	private int maxDim = 0;
	private boolean eliminaCaratteriSpeciali = false;

	public StringDocument(int maxDim) {
		super();
		
		if(maxDim < 0) {
			throw new IllegalArgumentException("La dimensione massima deve essere almeno 0. Valore: " + maxDim);
		}
		
		this.maxDim = maxDim;
	}

	public StringDocument(int maxDim, boolean eliminaCaratteriSpeciali) {
		super();
		
		if(maxDim < 0) {
			throw new IllegalArgumentException("La dimensione massima deve essere almeno 0. Valore: " + maxDim);
		}
		
		this.maxDim = maxDim;
		this.eliminaCaratteriSpeciali = eliminaCaratteriSpeciali;
	}

	
	public void insertString(
		int offset,
		String string,
		AttributeSet attributes)
		throws BadLocationException {

		if (string == null) {
			return;
		} else {
			string = string.toUpperCase();
			
			if(controllaCarattereInserito(string))
				return;
			
			String newValue;
			int length = getLength();
			if (length == 0) {
				newValue = string;
			} else {
				String currentContent = getText(0, length);
				StringBuffer currentBuffer = new StringBuffer(currentContent);
				currentBuffer.insert(offset, string);
				newValue = currentBuffer.toString();
			}
			// Questo Test serve ad evitare che si digitino pi caratteri di quelli permessi
			// Ovviamente in caso di copia ed incolla con questo test o si copia tutta la stringa o niente!
			if (newValue.length() <= maxDim) {
				super.insertString(offset, string, attributes);
			}
		}
	}
	
	private boolean controllaCarattereInserito(String carattere){	
		
		if(carattere != null && !carattere.equals("")){
			int arrCharNonAmmessi[] = new int[]{192,193,200,201,204,205,210,211,217,218,176,94,167};
			int charDigitato = (int)carattere.charAt(0);
			
			if (eliminaCaratteriSpeciali){
				if (charDigitato < 48 || (charDigitato > 57 && charDigitato < 65) || charDigitato > 90 ){
					return true;
				}
			}
			
			for (int i = 0; i < arrCharNonAmmessi.length; i++) {
				if(charDigitato == arrCharNonAmmessi[i])
					return true;
			}
			
			//System.out.println(charDigitato);		
		}				
				
		return false;
	}
}

