package it.finanze.dogane.accise.ui.validation;

import java.nio.charset.Charset;
import java.nio.charset.CharsetEncoder;

public class Validator {

/*************************************************************************************
	Per i sette caratteri con posizione di ordine pari:
	Tabella C  conversione dei caratteri con posizione di ordine pari
	A o 0 = 0	F o 5 = 5	K = 10	P = 15	U = 20
	B o 1 = 1	G o 6 = 6	L = 11	Q = 16	V = 21
	C o 2 = 2	H o 7 = 7	M = 12	R = 17	W = 22
	D o 3 = 3	I o 8 = 8	N = 13	S = 18	X = 23
	E o 4 = 4	J o 9 = 9	O = 14	T = 19	Y = 24
					Z = 25
	Per gli otto caratteri con posizione di ordine dispari:
	Tabella D conversione dei caratteri con posizione di ordine dispari
	A o 0 = 1	F o 5 = 13	K = 2 	P = 3	U = 16
	B o 1 = 0	G o 6 = 15	L = 4	Q = 6	V = 10
	C o 2 = 5	H o 7 = 17	M = 18	R = 8	W = 22
	D o 3 = 7	I o 8 = 19	N = 20	S = 12	X = 25
	E o 4 = 9	J o 9 = 21	O = 11	T = 14	Y = 24
					Z = 23
	2.      I valori numerici cos determinati vengono addizionati e la somma si divide per il numero 26. 
	Il carattere di controllo si ottiene convertendo il resto di tale divisione nel carattere alfabetico 
	ad esso corrispondente nella sotto indicata tabella:
	Tabella E  determinazione del check-digit
	0 = A	5 = F	10 = K	15 = P	20 = U
	1 = B	6 = G	11 = L	16 = Q	21 = V
	2 = C	7 = H	12 = M	17 = R	22 = W
	3 = D	8 = I	13 = N	18 = S	23 = X
	4 = E	9 = J	14 = O	19 = T	24 = Y
					25 = Z
 
************************************************************************************/

	private static final char[] TAB_D = {
		'B','A','K','P','L',
		'C','Q','D','R','E',
		'V','O','S','F','T',
		'G','U','H','M','I',
		'N','J','W','Z','Y','X'
	};
	
	private static final char[] TAB_D_NUMERIC = {
		'1','0',' ',' ',' ',
		'2',' ','3',' ','4',
		' ',' ',' ','5',' ',
		'6',' ','7',' ','8',
		' ','9'
	};

	private static final char[] TAB_C_E = {
		'A','B','C','D','E',
		'F','G','H','I','J',
		'K','L','M','N','O',
		'P','Q','R','S','T',
		'U','V','W','X','Y','Z'
	};
	
	private static final char[] TAB_C_NUMERIC = {
		'0','1','2','3','4',
		'5','6','7','8','9'
	};


	public static boolean isCodiceDittaForMetano(String codiceDitta) {
		boolean retvalue = isCodiceDitta(codiceDitta);
		
		retvalue = retvalue && "O".equals(codiceDitta.substring(2,3));
		
		return retvalue;
	}

	public static boolean isCodiceDittaForEnergiaElettrica(String codiceDitta) {
		boolean retvalue = isCodiceDitta(codiceDitta);
		
		retvalue = retvalue && "E".equals(codiceDitta.substring(2,3));
		
		return retvalue;
	}

	public static boolean isCodiceDittaForCarbonTax(String codiceDitta) {
		boolean retvalue = isCodiceDitta(codiceDitta);
		
		// Per Autotrasportatori  necessario solo un controllo formale

		return retvalue;
	}

	public static boolean isCodiceDitta(String codiceDitta)
	{
		if(codiceDitta.length()!=9) return false;


		String code = codiceDitta.substring(0, 2);
		String code2 = codiceDitta.substring(2, 3);
		String code3 = codiceDitta.substring(3, 8);
		String code4 = codiceDitta.substring(8);
	
		if(!isLiteral(code)) return false;
		if(!isNumeric(code3)) return false;
		if(!isLiteral(code4)) return false;
		if(!isCheckDigitValid(codiceDitta)) return false;
	
		return true;

	}

	private static boolean isCheckDigitValid(String codiceDitta) {
		boolean retvalue = true;
		char cd = computeCheckDigit(codiceDitta.substring(0, 8));
		char cd2 = codiceDitta.charAt(8);
		if(cd != cd2) {
			retvalue = false;
		}
		return retvalue;
	}

	private static char computeCheckDigit(String codiceDitta) {
		codiceDitta = "0000000" + codiceDitta;
		int sum = 0;
		for(int i=0; i < codiceDitta.length(); i+=2) {
			sum += getValorePosizioneDispari(codiceDitta.charAt(i));
			if(i+1 < codiceDitta.length()) {
				sum += getValorePosizionePari(codiceDitta.charAt(i+1));
			}
		}
		char retvalue = getCheckDigit(sum % 26);
		
		// System.out.println("Check Digit --> " + retvalue);
		return retvalue;
	}
	
	private static int getValorePosizionePari(char c) {
		char[] tabella = null;
		if(isNumeric(c)) {
			tabella = TAB_C_NUMERIC;
		} else {
			tabella = TAB_C_E;
		}
		
		for(int i = 0; i < tabella.length; i++) {
			if(tabella[i] == c) {
				return i;
			}
		}
		
		return 0;
	}

	private static int getValorePosizioneDispari(char c) {
		char[] tabella = null;
		if(isNumeric(c)) {
			tabella = TAB_D_NUMERIC;
		} else {
			tabella = TAB_D;
		}
		
		for(int i = 0; i < tabella.length; i++) {
			if(tabella[i] == c) {
				return i;
			}
		}
		
		return 0;
	}

	private static char getCheckDigit(int i) {
		return TAB_C_E[i];
	}


	private static boolean isNumeric(char c) {

		String digits = "0123456789";

		if(digits.indexOf(""+c) >= 0)
			return true;

		return false;
	}

	private static boolean isNumeric(String s) {

		for(int i = 0; i < s.length(); i++){
			char c = s.charAt(i);
			if(!isNumeric(c))
				return false;
		}

		return true;
	}

	private static boolean isLiteral(char c) {

		String digits = "ABCDEFGHIJKLMNOPQRSTUVWXYZ";

		if(digits.indexOf(""+c) >= 0)
			return true;

		return false;
	}

	private static boolean isLiteral(String s) {

		for(int i = 0; i < s.length(); i++){
			char c = s.charAt(i);
			if(!isLiteral(c))
				return false;
		}

		return true;
	}


	public static void main(String[] args) {
		String[] codici = {"CAO00001X", "CAE00001N"};

		for(int i=0; i < codici.length; i++) {
			String codiceDitta = codici[i];
			System.out.println("Elaborazione per : " + codiceDitta);
			if(isCodiceDitta(codiceDitta)) {
				System.out.println("Il codice risulta essere valido");
			} else {
				System.out.println("Il codice NON  valido");
				System.out.println("Il carattere di controllo dovrebbe essere :" + computeCheckDigit(codiceDitta.substring(0,8)));
			}
			System.out.println();
		}
	}
}