package it.finanze.dogane.accise.util;

import it.finanze.dogane.accise.model.Prospect;
import it.finanze.dogane.accise.ui.ConfirmFileDialog;
import it.finanze.dogane.accise.ui.DicFileFilter;


import java.awt.Component;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.math.BigInteger;
import java.util.LinkedList;
import java.util.List;

import javax.jnlp.FileContents;
import javax.jnlp.FileOpenService;
import javax.jnlp.FileSaveService;
import javax.jnlp.PrintService;
import javax.jnlp.ServiceManager;
import javax.jnlp.UnavailableServiceException;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;

public class ResourceAccess {

	private static FileOpenService fileOpenService = null;
	private static FileSaveService fileSaveService = null;
	private static PrintService printService = null;	
	public static final BigInteger MAX_SIZE_FILE_INTERCHANGE = BigInteger.valueOf(1390000); 
	//public static final BigInteger MAX_SIZE_FILE_INTERCHANGE = BigInteger.valueOf(3000); // size per test
	private FileContents contents = null;
	private File file = null;
	private Prospect prospect = null;
	private static String[] extensions = {"dic", "DIC"};

	static {
		try {
			fileOpenService = (FileOpenService)ServiceManager.lookup("javax.jnlp.FileOpenService");
			fileSaveService = (FileSaveService)ServiceManager.lookup("javax.jnlp.FileSaveService");
			printService = (PrintService)ServiceManager.lookup("javax.jnlp.PrintService");
		} catch(UnavailableServiceException e) {
			fileOpenService = null;
			fileSaveService = null;
			printService = null;
		}
	}

	public ResourceAccess(Prospect prospect) {
		if(prospect == null) {
			throw new IllegalArgumentException("Il prospetto non pu essere nullo.");
		}
		this.prospect = prospect;
	}

	/**
	 * @return
	 */
	public Prospect getProspect() {
		return prospect;
	}

	public String getFileName() {
		if(fileOpenService != null) {
			return getFileNameFromService();
		} else {
			return getFileNameStandardWay();
		}
	}

	/**
	 * @return
	 */
	private String getFileNameFromService() {
		if(contents != null) {
			try {
				return contents.getName();
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
		return null;
	}

	/**
	 * @return
	 */
	private String getFileNameStandardWay() {
		if(file != null) {
			return file.getName();
		}
		return null;
	}

	/**
	 * 
	 * @param component
	 * @return true se si  caricato qualcosa.
	 * @throws IOException
	 */
	public boolean loadProspect(Component component) throws IOException {
		boolean retvalue = false;
		InputStream is = openFile(component);
		if(is != null) {
			try {			
				prospect.leggiOggetto(is);
			}
			catch(IOException e) {
				throw e;
			}
			finally {
				is.close();
			}
			retvalue = true;
		}
		return retvalue;
	}

	public InputStream openFile(Component component) throws IOException {
		InputStream retvalue = null;
		if(fileOpenService != null) {
			retvalue = openFileFromService(component); // Se si lancia attraverso Java Web Start
		} else {
			retvalue = openFileStandardWay(component); // Se non si lancia attraverso Java Web Start
		}
		
		return retvalue;
	}

	public File openDirectory(Component component) throws IOException {
		File f = null;
		if(fileOpenService != null) {
			f = openDirectoryFromService(component); // Se si lancia attraverso Java Web Start
		} else {
			f = openDirectoryStandardWay(component); // Se non si lancia attraverso Java Web Start
		}
		return f;
	}

	private InputStream openFileFromService(Component component) throws IOException {
		InputStream retvalue = null;
		contents = fileOpenService.openFileDialog(null, extensions);

		if (contents!=null)
			retvalue = contents.getInputStream();

		return retvalue;
	}

	private File openDirectoryFromService(Component component) throws IOException {
		File f = null;
		contents = fileOpenService.openFileDialog("c:\\", null);
		f = new File(contents.getName());
		return f;
	}
	private InputStream openFileStandardWay(Component component) throws FileNotFoundException {
		InputStream retvalue = null;
		JFileChooser fileChooser = new JFileChooser();
		fileChooser.setFileFilter(new DicFileFilter());
		
		int userAction = fileChooser.showOpenDialog(component);

		if (userAction == JFileChooser.APPROVE_OPTION) {
			
			file = fileChooser.getSelectedFile();
			retvalue = new FileInputStream(file);
		}
		
		return retvalue;
	}

	private File openDirectoryStandardWay(Component component) throws FileNotFoundException {
		File f = null;
		JFileChooser fileChooser = new JFileChooser();
		fileChooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
		int userAction = fileChooser.showOpenDialog(component);
		if (userAction == JFileChooser.APPROVE_OPTION) {
			f = fileChooser.getSelectedFile();
		}
		return f;
	}
	/**
	 * 
	 * @param component
	 * @return true se si  scelto di salvare qualcosa
	 * @throws IOException
	 */
	public boolean saveFile(Component component) throws IOException {
		boolean retvalue = false;
		if(fileOpenService != null) {
			retvalue = saveFileFromService(component); // Se si lancia attraverso Java Web Start
		} else {
			retvalue = saveFileStandardWay(component); // Se non si lancia attraverso Java Web Start
		}
		return retvalue;
	}

	public boolean saveFileInterchange(Component component,String fileName) throws IOException {
		boolean retvalue = false;
		if(fileOpenService != null) {
			retvalue = saveFileFromServiceInterchange(component,fileName); // Se si lancia attraverso Java Web Start
		} else {
			retvalue = saveFileStandardWayInterchange(component,fileName); // Se non si lancia attraverso Java Web Start
		}
		return retvalue;
	}

	/**
	 * @param component
	 * @param prospect
	 */
	private boolean saveFileStandardWay(Component component) throws IOException {
		boolean retvalue = false;
		if(file == null) {
			retvalue = saveAsFileStandardWay(component);
		} else {
			FileOutputStream os = new FileOutputStream(file);

			prospect.scriviOggetto(os);
			os.close();
			retvalue = true;
		}
		return retvalue;
	}

	private boolean saveFileStandardWayInterchange(Component component,String fileName) throws IOException {
		boolean retvalue = false;

		ByteArrayOutputStream byteArrayOutputStream = new ByteArrayOutputStream();

		List listaOut=(List)prospect.scriviOggettoInterchange();
		if (listaOut.size()==1 && listaOut.get(0)!=null && (listaOut.get(0)) instanceof ByteArrayOutputStream){
			// in tal caso il file non necessita di essere spacchettato in n pacchetti
			byteArrayOutputStream=(ByteArrayOutputStream)listaOut.get(0);

			ConfirmFileDialog dialogInterchange=new ConfirmFileDialog(component,"Creazione file interchange",fileName,byteArrayOutputStream,"1 di 1",fileSaveService);
			dialogInterchange.setVisible(true);
			retvalue=dialogInterchange.getRetValue();
			/***
			// posso salvarlo con il nome specificato ....
			JFileChooser fileChooser = new JFileChooser();
			fileChooser.setFileFilter(new DicFileFilter());
			fileChooser.setSelectedFile(new File(fileName));		
			int userAction = fileChooser.showSaveDialog(component);
			if (userAction == JFileChooser.APPROVE_OPTION) {
				file = fileChooser.getSelectedFile();
				FileOutputStream fos = new FileOutputStream(file);
				byteArrayOutputStream.writeTo(fos);
				fos.close(); 
				retvalue = true;
			}
			***/
			byteArrayOutputStream.close();
		} else if (listaOut.size()>1) {
			// in tal caso il file  spacchettato
			int progressivo=1;
			int numSupporti=listaOut.size();
			for (int j=0; j < listaOut.size(); j++) {
				int numSupportiRimanenti=j+1;
				// calcolo il numero di supporti necessari per il salvataggio
				String numSupportiStr=numSupportiRimanenti + " di " + numSupporti;
				ByteArrayOutputStream baos=(ByteArrayOutputStream)listaOut.get(j);
				ConfirmFileDialog dialogInterchange=new ConfirmFileDialog(component,"Creazione file interchange",fileName+"_"+progressivo++,baos,numSupportiStr,fileSaveService);
				dialogInterchange.setVisible(true);
				retvalue=dialogInterchange.getRetValue();
				if (retvalue==false){
					return retvalue;
				}

				/***
				// posso salvarlo con il nome specificato ....
				JFileChooser fileChooser = new JFileChooser();
				fileChooser.setFileFilter(new DicFileFilter());
				fileChooser.setSelectedFile(new File(fileName+"_"+progressivo++));		
				int userAction = fileChooser.showSaveDialog(component);
				if (userAction == JFileChooser.APPROVE_OPTION) {
					file = fileChooser.getSelectedFile();
					FileOutputStream fos = new FileOutputStream(file);
					baos.writeTo(fos);
					fos.close(); 
					retvalue = true;
				}
				***/
				baos.close();
			}
		}
		return retvalue;
	}
	/**
	 * @param component
	 * @param prospect
	 */
	private boolean saveFileFromService(Component component) throws IOException {
		boolean retvalue = false;
		if(contents == null) {
			retvalue = saveAsFileFromService(component);
		} else {

			ByteArrayOutputStream byteArrayOutputStream = new ByteArrayOutputStream();
			prospect.scriviOggetto(byteArrayOutputStream);
			int sizeNeeded = byteArrayOutputStream.size();
			if (sizeNeeded  > contents.getMaxLength()){
				contents.setMaxLength(sizeNeeded);
			}
			OutputStream os = contents.getOutputStream(true);
			os.write(byteArrayOutputStream.toByteArray());

			os.close();
			byteArrayOutputStream.close();
			retvalue = true;
		}
		return retvalue;
	}

	private boolean saveFileFromServiceInterchange(Component component,String fileName) throws IOException {
		boolean retvalue = false;

		List listaOut=(List)prospect.scriviOggettoInterchange();
		if (listaOut.size()==1 && listaOut.get(0)!=null && (listaOut.get(0)) instanceof ByteArrayOutputStream){
			// in tal caso il file non necessita di essere spacchettato in n pacchetti
			retvalue = false;

			ByteArrayOutputStream os = new ByteArrayOutputStream();
			os=(ByteArrayOutputStream)listaOut.get(0);

			ConfirmFileDialog dialogInterchange=new ConfirmFileDialog(component,"Creazione file interchange",fileName,os,"1 di 1",fileSaveService);
			dialogInterchange.setVisible(true);
			retvalue=dialogInterchange.getRetValue();

			/***
			ByteArrayInputStream is = new ByteArrayInputStream(os.toByteArray());
			String suggestedName = fileName;
			JOptionPane.showMessageDialog(component, 
				"Si consiglia di rinominare il file come: "+fileName,
				fileName,
				JOptionPane.INFORMATION_MESSAGE);

			// dalle specifiche SUN il client potrebbe ignorare il nome suggerito !!!
			FileContents newContents = fileSaveService.saveFileDialog(null, extensions, is, suggestedName);
			if(newContents != null) {
				retvalue = true; 
			}
			is.close();
			****/

			os.close();
		} else if(listaOut.size()>1) {
			// in tal caso il file  spacchettato
			int progressivo=1;
			int numSupporti=listaOut.size();
			for (int j=0; j < listaOut.size(); j++) {
				retvalue = false;
				int numSupportiRimanenti=j+1;
				// calcolo il numero di supporti necessari per il salvataggio
				String numSupportiStr=numSupportiRimanenti + " di " + numSupporti;
	
				ByteArrayOutputStream baos = new ByteArrayOutputStream();
				baos=(ByteArrayOutputStream)listaOut.get(j);

				ConfirmFileDialog dialogInterchange=new ConfirmFileDialog(component,"Creazione file interchange",fileName+"_"+progressivo++,baos,numSupportiStr,fileSaveService);
				dialogInterchange.setVisible(true);
				retvalue=dialogInterchange.getRetValue();
				if (retvalue==false){
					return retvalue;
				}
				/***
				ByteArrayInputStream is = new ByteArrayInputStream(baos.toByteArray());
	
				String suggestedName = fileName+"_"+progressivo++;
				JOptionPane.showMessageDialog(component, 
					"Si consiglia di rinominare il file come: "+suggestedName,
					fileName,
					JOptionPane.INFORMATION_MESSAGE);
				// dalle specifiche SUN il client potrebbe ignorare il nome suggerito !!!
				FileContents newContents = fileSaveService.saveFileDialog(null, extensions, is, suggestedName);
				if(newContents != null) {
					retvalue = true; 
				}
				is.close();
				***/
				baos.close();
			}
		}
		return retvalue;
	}
	
	public boolean saveAsFile(Component component) throws IOException {
		boolean retvalue = false;
		if(fileOpenService != null) {
			retvalue = saveAsFileFromService(component); // Se si lancia attraverso Java Web Start
		} else {
			retvalue = saveAsFileStandardWay(component); // Se non si lancia attraverso Java Web Start
		}
		return retvalue;
	}
	public boolean saveAsFile(Component component,String fileName) throws IOException {
		boolean retvalue = false;
		if(fileOpenService != null) {
			retvalue = saveAsFileFromService(component,fileName); // Se si lancia attraverso Java Web Start
		} else {
			retvalue = saveAsFileStandardWay(component,fileName); // Se non si lancia attraverso Java Web Start
		}
		return retvalue;
	}

	/**
	 * @param component
	 * @param prospect
	 */
	private boolean saveAsFileStandardWay(Component component) throws IOException {
		boolean retvalue = false;
		JFileChooser fileChooser = new JFileChooser();
		fileChooser.setFileFilter(new DicFileFilter());
		
		if(file != null) {
			fileChooser.setSelectedFile(file);
		}
		
		int userAction = fileChooser.showSaveDialog(component);

		if (userAction == JFileChooser.APPROVE_OPTION) {
			
			file = fileChooser.getSelectedFile();
			
			
			/**************/
			//se l'estensione non  .dic, l'aggiungiamo noi
			if(!file.getName().endsWith(".dic")){
				
				String strPath = file.getAbsolutePath();
				file = new File(strPath+".dic");				
			}
						
			//se il file gi esiste
			if(file.exists()){
								
				int confirm = JOptionPane.showOptionDialog(null,
					"Il file "+file.getName()+" gi esiste. Sovrascriverlo ?", "Sovrascrivere File",
					JOptionPane.YES_NO_OPTION,JOptionPane.QUESTION_MESSAGE,null, null, null);
					if (confirm == 1)//No				
						return false;					
			}
			/**************/
			
			
			
			FileOutputStream os = new FileOutputStream(file);
			prospect.scriviOggetto(os);
			os.close();
			retvalue = true;
			
		}
		return retvalue;
	}
	/**
	 * @param component
	 * @param prospect
	 * @param fileName
	 */
	private boolean saveAsFileStandardWay(Component component,String fileName) throws IOException {
		boolean retvalue = false;

		ByteArrayOutputStream os = new ByteArrayOutputStream();

		Prospect tmpProspect = prospect;
		tmpProspect.scriviOggetto(os);

		ConfirmFileDialog dialogInterchange=new ConfirmFileDialog(component,"Invio file",fileName,os,"",fileSaveService);
		dialogInterchange.setVisible(true);
		retvalue=dialogInterchange.getRetValue();

		/***
		JFileChooser fileChooser = new JFileChooser();
		//fileChooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
		fileChooser.setFileFilter(new DicFileFilter());
		fileChooser.setSelectedFile(new File(fileName));		

		int userAction = fileChooser.showSaveDialog(component);

		if (userAction == JFileChooser.APPROVE_OPTION) {
			
			file = fileChooser.getSelectedFile();
			FileOutputStream os = new FileOutputStream(file);
			prospect.scriviOggetto(os);
			os.close();
			retvalue = true;
		}
		***/
		os.close();
		return retvalue;
	}

	/**
	 * @param component
	 * @param prospect
	 */
	private boolean saveAsFileFromService(Component component) throws IOException {
		boolean retvalue = false;
		
		ByteArrayOutputStream os = new ByteArrayOutputStream();
		prospect.scriviOggetto(os);
		ByteArrayInputStream is = new ByteArrayInputStream(os.toByteArray());
		
		String suggestedName = null;
		if(contents != null) {
			suggestedName = contents.getName();
		}
		
		FileContents newContents = fileSaveService.saveFileDialog(null, extensions, is, suggestedName);
		if(newContents != null) {
			contents = newContents;
			retvalue = true; 
		}

		return retvalue;
	}

	/**
	 * @param component
	 * @param prospect
	 * @param fileName
	 */
	private boolean saveAsFileFromService(Component component,String fileName) throws IOException {
		boolean retvalue = false;
		
		ByteArrayOutputStream os = new ByteArrayOutputStream();

		Prospect tmpProspect = prospect;
		tmpProspect.scriviOggetto(os);

		ConfirmFileDialog dialogInterchange=new ConfirmFileDialog(component,"Invio file",fileName,os,"",fileSaveService);
		dialogInterchange.setVisible(true);
		retvalue=dialogInterchange.getRetValue();

		/***		
		ByteArrayInputStream is = new ByteArrayInputStream(os.toByteArray());
		String suggestedName = fileName;
		JOptionPane.showMessageDialog(component, 
			"Si consiglia di rinominare il file come: "+fileName,
			fileName,
			JOptionPane.INFORMATION_MESSAGE);
		// dalle specifiche SUN il client potrebbe ignorare il nome suggerito !!!
		FileContents newContents = fileSaveService.saveFileDialog(null, extensions, is, suggestedName);
		if(newContents != null) {
			contents = newContents;
			retvalue = true; 
		}
		***/
		os.close();
		return retvalue;
	}

}
